



where uhd_cal_rx_iq_balance
IF %ERRORLEVEL% NEQ 0 exit /B 1
where uhd_cal_tx_dc_offset
IF %ERRORLEVEL% NEQ 0 exit /B 1
where uhd_cal_tx_iq_balance
IF %ERRORLEVEL% NEQ 0 exit /B 1
where uhd_config_info
IF %ERRORLEVEL% NEQ 0 exit /B 1
where uhd_find_devices
IF %ERRORLEVEL% NEQ 0 exit /B 1
where uhd_image_loader
IF %ERRORLEVEL% NEQ 0 exit /B 1
where uhd_usrp_probe
IF %ERRORLEVEL% NEQ 0 exit /B 1
uhd_config_info --print-all
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
